import cv2
import numpy as np
import socket

#INPUT SEND DATA
IP = '10.68.209.77' #paste your own IP here
port = 6002
#----------------------------------------


# capture video
cap = cv2.VideoCapture(0, cv2.CAP_DSHOW)

#segment one color (or more)
#setting color limits (for red)

colorLowL = np.array([0, 100, 20], np.uint8)
colorUpL = np.array([10, 255, 255], np.uint8)

# define the kernel
kernel = np.ones((5, 5), "uint8")

# define the color name
cName = 'red'
cOut = (0, 0, 255)
run = True
minArea = 1000

try:
    while (cap.isOpened()):
        ret, RGBframe = cap.read()

        #FIRST, COLOUR SEGMENTATION
        width = RGBframe.shape[1]
        height = RGBframe.shape[0]

        frame = (width, height)

        HSVframe = cv2.cvtColor(RGBframe, cv2.COLOR_BGR2HSV)

        # define the color mask
        colorMask = cv2.inRange(HSVframe, colorLowL, colorUpL)

        #
        colorMask = cv2.dilate(colorMask, kernel)
        colorRes = cv2.bitwise_and(RGBframe, RGBframe, mask=colorMask)

        #SECOND, GENERATE CONTOURS FOR THE SEGMENTED COLOUR ONLY
        gray = cv2.cvtColor(colorRes, cv2.COLOR_BGR2GRAY)
        blurred = cv2.GaussianBlur(gray, (5, 5), 0)
        thresh = cv2.threshold(gray, 0, 255, cv2.THRESH_BINARY)[1]

        contours, hierarchy = cv2.findContours(thresh, cv2.RETR_LIST, cv2.CHAIN_APPROX_SIMPLE)

        print(contours)

        areas = []
        cir_cent = []
        tri_cent = []
        rect_cent = []
        u_cent = []
        items = []
        circles = []
        triangles = []
        rectangles = []
        unidentified = []

        for c in contours:

            area = cv2.contourArea(c)

            epsilon = 0.04 * cv2.arcLength(c, True)
            approx = cv2.approxPolyDP(c, epsilon, True)
            x = approx.ravel()[0]
            y = approx.ravel()[1] - 5

            if (area > minArea):
                detection = True
                items.append(c)

                approx = cv2.approxPolyDP(c, 0.01 * cv2.arcLength(c, True), True)
                cv2.drawContours(RGBframe, [approx], 0, (0, 0, 0), 1)
                areas.append(area)

                if (len(approx) > 5):
                    u= cv2.putText(RGBframe, "not sure", (x, y), cv2.FONT_HERSHEY_COMPLEX, 0.5, (0, 0, 0))
                    unidentified.append(u)

                    I = cv2.moments(c)
                    iX = int(I["m10"] / I["m00"])
                    iY = int(I["m01"] / I["m00"])

                    ci = [(iX, iY)]
                    u_cent.append(ci)


                if (len(approx) == 5):
                    cir = cv2.putText(RGBframe, "circle", (x, y), cv2.FONT_HERSHEY_COMPLEX, 0.5, (0, 0, 0))
                    detection = True
                    circles.append(cir)

                    M = cv2.moments(c)
                    cX = int(M["m10"] / M["m00"])
                    cY = int(M["m01"] / M["m00"])

                    cc = [(cX, cY)]
                    cir_cent.append(cc)

                if (len(approx) == 4):
                    cv2.putText(RGBframe, "rectangle", (x, y), cv2.FONT_HERSHEY_COMPLEX, 0.5, (0, 0, 0))

                    N = cv2.moments(c)
                    rX = int(N["m10"] / N["m00"])
                    rY = int(N["m01"] / N["m00"])

                    rectCent = [(rX, rY)]

                    rect_cent.append(rectCent)

                if (len(approx) == 3):
                    cv2.putText(RGBframe, "triangle", (x, y), cv2.FONT_HERSHEY_COMPLEX, 0.5, (0, 0, 0))
                    U = cv2.moments(c)
                    tX = int(U["m10"] / U["m00"])
                    tY = int(U["m01"] / U["m00"])

                    trgCent = [(tX, tY)]

                    tri_cent.append(trgCent)


        cv2.imshow("contour approximation", RGBframe)
        if cv2.waitKey(10) & 0xFF == ord('q'):
            break

        # LIVE DATA STREAMING MODULE

        n = 'contour and shape approximation'  # input script color here
        d = detection
        c = 'circles.{}/triangles.{}/rectangles.{}/unidentified{}'.format(cir_cent, tri_cent, rect_cent, u_cent )
        e = contours
        a = areas
        s = 'circles.{}/triangles.{}/rectangles.{}/unidentified{}'.format(len(circles), len(triangles), len(rectangles), len(unidentified))
        i = len(items)
        f = frame

        template = str(
            'name:{}|detection:{}|center:{}|edge:{}|area:{}|shape:{}|items:{}|frame:{}'.format(n, d, c, e, a, s, i, f))


        def UDP_client(IP, port, message):
            sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            sock.sendto(bytes(message, "utf-8"), (IP, port))


        UDP_client(IP, port, template)

    cap.release()
    cv2.destroyAllWindows()

except Exception as e:
    print(e)

