import cv2
import numpy as np
import socket


IP = '192.168.61.232'  # your IP
port = 6002  # your port

# capture video
cap = cv2.VideoCapture(0, cv2.CAP_DSHOW)


#COLOR BOUNDARIES
#red
rlow = np.array([0, 100, 20], np.uint8)
rhigh = np.array([10, 255, 255], np.uint8)
rn = "red"
ro = (0, 0, 255)

#blue
blow = np.array([100,150,0], np.uint8)
bhigh = np.array([140,255,255], np.uint8)
bn = "blue"
bo = (220, 73, 96)

#green
glow = np.array([25, 52, 72], np.uint8)
ghigh = np.array([102, 255, 255], np.uint8)
gn = "green"
go = (66, 245, 158)


#OTHER VARIABLES
kernel = np.ones((5, 5), "uint8")
minArea = 1000


try:
    while (cap.isOpened()):
        ret, RGBframe = cap.read()

        # convert colour mode > HSV (hue saturation value) instead of RGB
        # and store the value into a new variable called "HSVframe"
        width = RGBframe.shape[1]
        height = RGBframe.shape[0]
        frame = (width, height)

        HSVframe = cv2.cvtColor(RGBframe, cv2.COLOR_BGR2HSV)

        # DEFINE MASKS
        r_mask = cv2.inRange(HSVframe, rlow, rhigh)
        b_mask = cv2.inRange(HSVframe, blow, bhigh)
        g_mask = cv2.inRange(HSVframe, glow, ghigh)

        # DETECTION

        # Red Color
        r_mask = cv2.dilate(r_mask, kernel)
        r_res = cv2.bitwise_and(RGBframe, RGBframe, mask=r_mask)

        contours, hierarchy = cv2.findContours(r_mask, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)

        r_areas = []
        r_centers = []


        for pic, contour in enumerate(contours):
            area = cv2.contourArea(contour)

            if (area > minArea):
                detection = True
                r_areas.append(area)

                #calculate centers
                rM = cv2.moments(contour)
                rx = int(rM['m10'] / rM['m00'])
                ry = int(rM['m01'] / rM['m00'])

                r_center = (rx, ry)
                r_centers.append(r_center)

                x, y, w, h = cv2.boundingRect(contour)
                RGBframe = cv2.rectangle(RGBframe, (x, y), (x + w, y + h), ro, 2)
                cv2.putText(RGBframe, rn, (x, y), cv2.FONT_HERSHEY_SIMPLEX, 1.0, ro)

        red_items = len(r_areas)


        # Blue Color
        b_mask = cv2.dilate(b_mask, kernel)
        b_res = cv2.bitwise_and(RGBframe, RGBframe, mask=b_mask)

        contours, hierarchy = cv2.findContours(b_mask, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)

        b_areas = []
        b_centers = []

        for pic, contour in enumerate(contours):
            area = cv2.contourArea(contour)

            if (area > minArea):
                detection = True
                b_areas.append(area)

                # calculate centers
                bM = cv2.moments(contour)
                bx = int(bM['m10'] / bM['m00'])
                by = int(bM['m01'] / bM['m00'])

                b_center = (bx, by)
                b_centers.append(r_center)

                x, y, w, h = cv2.boundingRect(contour)
                RGBframe = cv2.rectangle(RGBframe, (x, y), (x + w, y + h), bo, 2)
                cv2.putText(RGBframe, bn, (x, y), cv2.FONT_HERSHEY_SIMPLEX, 1.0, bo)


        blue_items = len(b_areas)

        # Green Color
        g_mask = cv2.dilate(g_mask, kernel)
        g_res = cv2.bitwise_and(RGBframe, RGBframe, mask=g_mask)

        contours, hierarchy = cv2.findContours(g_mask, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)


        #print(gcontours)
        #print(len(gcontours))
        g_areas = []
        g_centers = []

        for pic, contour in enumerate(contours):
            area = cv2.contourArea(contour)

            if (area > minArea):
                detection = True
                g_areas.append(area)

                # calculate centers
                gM = cv2.moments(contour)
                gx = int(gM['m10'] / gM['m00'])
                gy = int(gM['m01'] / gM['m00'])

                g_center = (gx, gy)
                g_centers.append(g_center)

                x, y, w, h = cv2.boundingRect(contour)
                RGBframe = cv2.rectangle(RGBframe, (x, y), (x + w, y + h), go, 2)
                cv2.putText(RGBframe, gn, (x, y), cv2.FONT_HERSHEY_SIMPLEX, 1.0, go)


        print(g_areas)
        green_items = len(g_areas)

        cv2.imshow("multiple color detection", RGBframe)
        if cv2.waitKey(10) & 0xFF == ord('q'):
            break

        # LIVE DATA STREAMING MODULE

        n = 'multiple color detection'  # input script color here
        d = detection
        c = 'red.{}/green.{}/blue.{}'.format(r_centers, g_centers, b_centers) #all centers in x y format
        e = 'no edge'
        a = 'red.{}/green.{}/blue.{}'.format(r_areas, g_areas, b_areas) #all areas
        s = 'no shape'
        i = 'red.{}/green.{}/blue.{}'.format(red_items, green_items, blue_items) #number of items
        f = frame
        template = str(
            'name:{}|detection:{}|center:{}|edge:{}|area:{}|shape:{}|items:{}|frame:{}'.format(n, d, c, e, a, s, i, f))


        def UDP_client(IP, port, message):
            sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            sock.sendto(bytes(message, "utf-8"), (IP, port))


        UDP_client(IP, port, template)

    cap.release()
    cv2.destroyAllWindows()

except Exception as e:
    print(e)



